/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package fr.amap.lidar.amapvox.voxelisation.tls;

import javax.vecmath.Matrix3d;
import javax.vecmath.Matrix4d;
import fr.amap.lidar.amapvox.commons.VoxelSpaceInfos;
import fr.amap.lidar.amapvox.util.AVoxTask;
import fr.amap.lidar.amapvox.voxelisation.Voxelization;
import fr.amap.lidar.amapvox.voxelisation.configuration.TLSVoxCfg;
import fr.amap.lidar.amapvox.voxelisation.configuration.params.VoxelParameters;
import java.io.File;
import org.apache.log4j.Logger;

/**
 *
 * @author calcul
 */
public abstract class TLSVoxelisation extends AVoxTask {

    private final static Logger LOGGER = Logger.getLogger(TLSVoxelisation.class);

    protected int nbVoxelisationFinished;
    protected File inputFile;
    protected Voxelization voxelAnalysis;
    protected Matrix4d transformation;
    protected VoxelParameters parameters;
    protected File outputFile;
    protected final TLSVoxCfg cfg;

    public TLSVoxelisation(File file, int ncpu) {

        super(file, ncpu);
        cfg = new TLSVoxCfg();
    }

    public TLSVoxelisation(TLSVoxCfg cfg, int ncpu) {
        super(null, ncpu);
        this.cfg = cfg;
    }

    @Override
    public void init() throws Exception {

        if (null != file) {
            cfg.readConfiguration(file);
        }
        this.inputFile = cfg.getInputFile();
        this.parameters = cfg.getVoxelParameters();
        parameters.infos.setType(VoxelSpaceInfos.Type.TLS);
        this.outputFile = cfg.getOutputFile();

        nbVoxelisationFinished = 0;

        // Transformation matrices
        
        Matrix4d pop = cfg.getPopMatrix();
        if (null == pop) {
            pop = new Matrix4d();
            pop.setIdentity();
        }
        Matrix4d vop = cfg.getVopMatrix();
        if (null == vop) {
            vop = new Matrix4d();
            vop.setIdentity();
        }

        Matrix4d popvop = new Matrix4d();
        popvop.mul(pop, vop);
        transformation = new Matrix4d();
        transformation.mul(pop, vop);
        if (null != cfg.getSopMatrix()) {
            transformation.mul(cfg.getSopMatrix());
        }

        voxelAnalysis = new Voxelization(cfg);
    }

    @Override
    public void setCancelled(boolean cancelled) {
        super.setCancelled(cancelled);
        // propagate cancellation to Voxelization.java
        voxelAnalysis.setCancelled(cancelled);
    }
}
